#include <stdarg.h>
#include <string.h>
#include "ggi_widget.h"
#include "ggi_widget_internal.h"

struct containerlinedata {

	ggi_widget_size_t drawpos,drawsize;
	int doopt;
	int padx,pady;
};

static int containerline_getsize(ggi_widget_t self) {

	struct containerlinedata *mine=self->priv;
	ggi_widget_t curchild;	
	int x;

	self->min.x=self->min.y=0;
	self->opt.x=self->opt.y=0;

	for(x=0;(curchild=self->getchild(self,x));x++) {
		self->min.x+=curchild->min.x+mine->padx*2;
		if (self->min.y<curchild->min.y) {
			self->min.y=curchild->min.y;
		}
		self->opt.x+=curchild->opt.x+mine->padx*2;
		if (self->opt.y<curchild->opt.y) {
			self->opt.y=curchild->opt.y;
		}
	}
	self->min.y+=2*mine->pady;
	self->opt.y+=2*mine->pady;
	return 0;
}

static int ggiWidgetContainerLineRedraw(ggi_widget_t self,int flush) {

	struct containerlinedata *mine=self->priv;
	int rc;

	ggiWidgetDrawBorder(self,&mine->drawpos,&mine->drawsize);
	rc=ggiWidgetRedrawChildren(self);
	if (flush) {
		ggiFlushRegion(self->vis,self->pos.x,self->pos.y,self->size.x,self->size.y);
	}
	return rc;
}

static int ggiWidgetContainerLineVischanged(ggi_widget_t self) {

	struct containerlinedata *mine=self->priv;
	ggi_widget_t curchild;
	int mysizeremain,x,numchilds,dx,add;

	ggiWidgetGetPlacement(self,&mine->drawpos,&mine->drawsize,&mine->doopt,0,0);

	mysizeremain   =mine->drawsize.x - 
			(mine->doopt ? self->opt.x : self->min.x);

	for(numchilds=0;self->getchild(self,numchilds);numchilds++);

	for(x=dx=0;x<numchilds;x++) {
		curchild=self->getchild(self,x);
		curchild->vis=self->vis;
		curchild->pos.x =self->pos.x+mine->drawpos.x+dx;
		curchild->pos.y =self->pos.y+mine->drawpos.y;
		curchild->size.y=mine->drawsize.y;
		if (mine->doopt) {
			curchild->size.x=curchild->opt.x;
		} else {
			curchild->size.x=curchild->min.x;
		}
		add=mysizeremain/(numchilds-x);
		curchild->size.x+=add+2*mine->padx;
		mysizeremain-=add;
		dx+=curchild->size.x;
		curchild->vischanged(curchild);
	}

	return 0;
}

static int ggiWidgetContainerLineChildchanged(ggi_widget_t self,ggi_widget_t child) {

	containerline_getsize(self);	/* Recalc my size */
	
	if (self->parent) self->parent->childchanged(self->parent,self);
	else              self->vischanged(self);

	return 0;
}

ggi_widget_t ggiWidgetCreateContainerLine(va_list ap) {

	ggi_widget_t newid;
	ggi_widget_t parms;
	
	struct containerlinedata *newbut;

	newid=ggiWidgetAllocate();
	if (!newid) return NULL;

	if (NULL==(newbut=malloc(sizeof(*newbut)))) {
		free(newid);
		return NULL;
	}
	newbut->padx=va_arg(ap,int);
	newbut->pady=va_arg(ap,int);
	newid->priv=newbut;

	newid->redraw      =ggiWidgetContainerLineRedraw;
	newid->vischanged  =ggiWidgetContainerLineVischanged;
	newid->childchanged=ggiWidgetContainerLineChildchanged;

	newid->getchild   =ggiWidgetChildlistGet;
	newid->linkchild  =ggiWidgetChildlistLink;
	newid->unlinkchild=ggiWidgetChildlistUnlink;

	while( (parms=va_arg(ap,ggi_widget_t)) ) {
		newid->linkchild(newid,LAST_CHILD,parms);
	}

	containerline_getsize(newid);

	return newid;
}
